
local mod = ...



-- Check water level
mod:registerBehaviorTreeNode({
    Name = "BEHAVIOR_NODE_IRRIGATE_CARRY_CHECKWATERLEVEL",
    VariableList = {
		AgentData = "BEHAVIOR_TREE_DATA_AGENT",
    },

    Update = function(self, instance)

        local compAgent = nil
        compAgent = self.AgentData.Agent

        local compWork = nil
        compWork = compAgent:getOwner():getComponent("COMP_VILLAGER"):getJobInstance().Workplace

        if compWork then

            local compTower = nil
            compTower = compWork:getOwner():findFirstParentWithComponent("COMP_IRRIGATION_TOWER")
            if (compTower) then

                if compTower.WaterLevel >= 10 then
                    return BEHAVIOR_TREE_NODE_RESULT.FALSE

                else
                    return BEHAVIOR_TREE_NODE_RESULT.TRUE

                end

            else
                return BEHAVIOR_TREE_NODE_RESULT.FALSE

            end

        else
            return BEHAVIOR_TREE_NODE_RESULT.FALSE

        end

    end

})


-- Update water level
mod:registerBehaviorTreeNode({
    Name = "BEHAVIOR_NODE_IRRIGATION_UPDATEWATERLEVEL",
    VariableList = {
		AgentData = "BEHAVIOR_TREE_DATA_AGENT",
    },

    Update = function(self, instance)

        local compAgent = nil
        compAgent = self.AgentData.Agent

        local compWork = nil
        compWork = compAgent:getOwner():getComponent("COMP_VILLAGER"):getJobInstance().Workplace

        if compWork then

            local compTower = nil
            compTower = compWork:getOwner():findFirstParentWithComponent("COMP_IRRIGATION_TOWER")
            if (compTower) then

                if compTower.WaterLevel >= 10 then
                    return BEHAVIOR_TREE_NODE_RESULT.FALSE

                else
                    compTower.WaterLevel = compTower.WaterLevel + 1
                    compTower:drawWater()
                    return BEHAVIOR_TREE_NODE_RESULT.TRUE

                end

            else
                return BEHAVIOR_TREE_NODE_RESULT.FALSE

            end

        else
            return BEHAVIOR_TREE_NODE_RESULT.FALSE

        end

    end

})


-- Transport setup
mod:registerAsset({
    DataType = "INTERACTIVE_LOCATION_SETUP",
    Id = "INTERACTIVE_LOCATION_SETUP_IRRIGATION",
    PurposeList = {
        "REQUEST",
        "WORK",
    },
    CharacterSetup = {
        MaleFrontList = {
            "PREFAB_RESOURCE_VISUAL_CRATE",
        },
        FemaleFrontList = {
            "PREFAB_RESOURCE_VISUAL_CRATE",
        },
        WalkAnimation = "WALKING",
    },
})



-- BEHAVIOR tree ------------------------------------------------------------------------------------------------
mod:registerBehaviorTree({
    Id = "BEHAVIOR_IRRIGATION_CARRY",
    VariableList = {
        {
            Name = "AgentData",
            DataType = "BEHAVIOR_TREE_DATA_AGENT",
            IsPublic = false,
            InitialValue = {}
        },
		{
			Name = "DoJobTimer",
			DataType = "BEHAVIOR_TREE_DATA_WAIT",
			IsPublic = false,
			InitialValue = {
				TimeToWait = 0,
				Animation = AGENT_ANIMATION.IDLE,
				SetIdleAfterWait = true,
			}
		},
		{
			Name = "WorkplacePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true,
			}
		},
		{
			Name = "TransportData",
			DataType = "BEHAVIOR_TREE_DATA_RESOURCE_TRANSPORT",
			IsPublic = false,
            InitialValue = {
                TransportSetup = "INTERACTIVE_LOCATION_SETUP_IRRIGATION",
            }
		},
		{
			Name = "ResourceProducedList",
			DataType = "BEHAVIOR_TREE_DATA_RESOURCE_PRODUCTION",
			IsPublic = false,
		},
		{
			Name = "WorkLoop",
			DataType = "BEHAVIOR_TREE_DATA_LOOP",
			IsPublic = true,
			InitialValue = {
				LoopCount = 0,
				Duration = 120,
				IsInfinite = false,
				IsDuration = true,
			}
		},
    },
	Root = {
		Name = "WorkingGlobalSequencer",
		Type = "SEQUENCER",
		Children = {
            {
                Name = "HasWorkplace",
                Type = "IS_WORKPLACE_AVAILABLE",
                AgentData = "AgentData"
            },
            {
                Name = "CheckWaterLevel",
                Type = "BEHAVIOR_NODE_IRRIGATE_CARRY_CHECKWATERLEVEL",
                AgentData = "AgentData",
            },
            {
                Name = "NeedFilledSelector",
                Type = "SELECTOR",
                Children = {
                    {
                        Name = "NeedFilledInverter",
                        Type = "INVERTER",
                        Child = {
                            Name = "IsMyNeedFilled",
                            Type = "IS_WORKPLACE_NEED_RESOURCE",
                            AgentData = "AgentData",
                        }
                    },
                    {
                        Name = "FetchResourceSequencer",
                        Type = "SEQUENCER",
                        Children = {
                            {
                                Name = "FindResourceForWorkplace",
                                Type = "FIND_RESOURCE_FOR_WORKPLACE",
                                AgentData = "AgentData",
                                TransportData = "TransportData",
                            },
                            {
                                Name = "Transport",
                                Type = "BEHAVIOR_TRANSPORT",
                                AgentData = "AgentData",
                                TransportData = "TransportData",
                            },
                        },
                    },
                },
            },
			{
				Name = "WorkLoopRepeater",
				Type = "REPEAT_UNTIL_FAIL",
				RepeatData = "WorkLoop",
				Child = {
					Name = "WorkLoopSequencer",
					Type = "SEQUENCER",
					Children = {
						{
							Name = "SetupWork",
							Type = "SETUP_WORK",
							AgentData = "AgentData",
							TimeToWait = "DoJobTimer",
							WorkPosition = "WorkplacePosition",
						},
                        {
							Name = "GoToWork",
							Type = "GO_TO",
							AgentData = "AgentData",
							Destination = "WorkplacePosition",
							BuildingPathType = "",
							AnimationData = "",
							AnimationSpeedMultiplier = "",
						},
						{
							Name = "UpdateWaterLevel",
							Type = "BEHAVIOR_NODE_IRRIGATION_UPDATEWATERLEVEL",
							AgentData = "AgentData",
						},
						{
							Name = "ProduceResource",
							Type = "PRODUCE_RESOURCE",
							AgentData = "AgentData",
							TimeToWait = "DoJobTimer",
							ResourceProducedList = "ResourceProducedList",
						},
						{
							Name = "GettingXp",
							Type = "GIVE_JOB_XP",
							AgentData = "AgentData",
							ShouldReceiveXp = "",
						},
					},
				},
			},
		},
	},
})

