
local mod = ...



-- Check water level
mod:registerBehaviorTreeNode({
    Name = "BEHAVIOR_NODE_IRRIGATE_DIG_CHECKWATERLEVEL",
    VariableList = {
		AgentData = "BEHAVIOR_TREE_DATA_AGENT",
    },

    Update = function(self, instance)

        local compAgent = nil
        compAgent = self.AgentData.Agent


        local compWork = nil
        compWork = compAgent:getOwner():getComponent("COMP_VILLAGER"):getJobInstance().Workplace

        if compWork then

            local compTower = nil
            compTower = compWork:getOwner():findFirstParentWithComponent("COMP_IRRIGATION_TOWER")
            if (compTower) then

                if compTower.WaterLevel > 0 then
                    return BEHAVIOR_TREE_NODE_RESULT.TRUE

                else
                    return BEHAVIOR_TREE_NODE_RESULT.FALSE

                end

            else
                return BEHAVIOR_TREE_NODE_RESULT.FALSE

            end

        else
            return BEHAVIOR_TREE_NODE_RESULT.FALSE

        end

    end

})


-- Reduce water level
mod:registerBehaviorTreeNode({
    Name = "BEHAVIOR_NODE_IRRIGATE_DIG_REDUCEWATERLEVEL",
    VariableList = {
		AgentData = "BEHAVIOR_TREE_DATA_AGENT",
    },

    Update = function(self, instance)

        local compAgent = nil
        compAgent = self.AgentData.Agent


        local compWork = nil
        compWork = compAgent:getOwner():getComponent("COMP_VILLAGER"):getJobInstance().Workplace

        if compWork then

            local compTower = nil
            compTower = compWork:getOwner():findFirstParentWithComponent("COMP_IRRIGATION_TOWER")
            if (compTower) then

                if compTower.WaterLevel > 0 then
                    compTower.WaterLevel = compTower.WaterLevel - 1
                    compTower:drawWater()
                    return BEHAVIOR_TREE_NODE_RESULT.TRUE

                else
                    return BEHAVIOR_TREE_NODE_RESULT.FALSE

                end

            else
                return BEHAVIOR_TREE_NODE_RESULT.FALSE

            end

        else
            return BEHAVIOR_TREE_NODE_RESULT.FALSE

        end

    end

})


-- BEHAVIOR tree --------------------------------------------------------------------------------------------------
mod:registerBehaviorTree({
	Id = "BEHAVIOR_IRRIGATION_DIG",
	VariableList = {
		{
			Name = "AgentData",
			DataType = "BEHAVIOR_TREE_DATA_AGENT",
			IsPublic = true,
			InitialValue = {}
		},
        {
			Name = "GatheringData",
			DataType = "BEHAVIOR_TREE_DATA_GATHERING",
			IsPublic = false,
			InitialValue = {}
		},
		{
			Name = "DoJobTimer",
			DataType = "BEHAVIOR_TREE_DATA_WAIT",
			IsPublic = false,
			InitialValue = {
				TimeToWait = 0,
				Animation = AGENT_ANIMATION.MINE,
				SetIdleAfterWait = false,
			},
		},
		{
			Name = "DepositResourceTimer",
			DataType = "BEHAVIOR_TREE_DATA_WAIT",
			IsPublic = false,
			InitialValue = {
				TimeToWait = 0,
				Animation = AGENT_ANIMATION.IDLE,
				SetIdleAfterWait = false,
			},
		},
		{
			Name = "WorkplacePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true
			}
		},
        {
			Name = "ResourcePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true
			}
		},
        {
			Name = "AroundResourcePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true
			}
		},
        {
			Name = "ShouldLookAtResourcePosition",
			DataType = "BEHAVIOR_TREE_DATA_BOOL",
			IsPublic = false,
			InitialValue = {
				Value = true
			}
		},
        {
			Name = "ResourceProducedList",
			DataType = "BEHAVIOR_TREE_DATA_RESOURCE_PRODUCTION",
			IsPublic = false,
		},
        {
			Name = "ShouldReceiveXp",
			DataType = "BEHAVIOR_TREE_DATA_BOOL",
			IsPublic = false,
			InitialValue = {
				Value = false
			}
		},
		{
			Name = "WorkLoop",
			DataType = "BEHAVIOR_TREE_DATA_LOOP",
			IsPublic = true,
			InitialValue = {
				LoopCount = 1,
				Duration = 0,
				IsInfinite = false,
				IsDuration = false
			}
		},
	},
	Root = {
        Name = "WorkLoopRepeater",
        Type = "REPEAT",
        RepeatData = "WorkLoop",
        Child = {
            Name = "WorkLoopSequencer",
            Type = "SEQUENCER",
            Children = {
                {
                    Name = "HasWorkplace",
                    Type = "IS_WORKPLACE_AVAILABLE",
                    AgentData = "AgentData"
                },
                {
                    Name = "SetWorkplaceDestination",
                    Type = "SET_WORKPLACE_AS_DESTINATION",
                    AgentData = "AgentData",
                    WorkplacePosition = "WorkplacePosition",
                },
                {
                    Name = "GoToWorkplaceStart",
                    Type = "GO_TO",
                    AgentData = "AgentData",
                    Destination = "WorkplacePosition",
                    BuildingPathType = "",
                    AnimationData = "",
                    AnimationSpeedMultiplier = ""
                },
                {
                    Name = "CheckWaterLevel",
                    Type = "BEHAVIOR_NODE_IRRIGATE_DIG_CHECKWATERLEVEL",
                    AgentData = "AgentData",
                },
                {
                    Name = "GatherRequest",
                    Type = "GATHER_REQUEST",
                    AgentData = "AgentData",
                    GatheringData = "GatheringData",
                    GatherInOneStepLoopData = "WorkLoop"
                },
                {
                    Name = "FetchGatherable",
                    Type = "FETCH_NEXT_GATHERABLE",
                    AgentData = "AgentData",
                    GatheringData = "GatheringData",
                    TimeToWait = "DoJobTimer",
                    ResourcePosition = "ResourcePosition",
                    AroundResourcePosition = "AroundResourcePosition",
                    ShouldLookAtResourcePosition = "ShouldLookAtResourcePosition"
                },
                {
                    Name = "DisablePaths",
                    Type = "DISABLE_PATH_TRACING",
                    AgentData = "AgentData"
                },
                {
                    Name = "ReduceWaterLevel",
                    Type = "BEHAVIOR_NODE_IRRIGATE_DIG_REDUCEWATERLEVEL",
                    AgentData = "AgentData",
                },
                {
                    Name = "GoToGatherable",
                    Type = "GO_TO",
                    AgentData = "AgentData",
                    Destination = "ResourcePosition",
                    BuildingPathType = "",
                    AnimationData = "",
                    AnimationSpeedMultiplier = ""
                },
                {
                    Name = "LookAtGatherable",
                    Type = "LOOK_AT",
                    AgentData = "AgentData",
                    Destination = "ResourcePosition"
                },
                {
                    Name = "GatherResource",
                    Type = "GATHER_RESOURCE",
                    AgentData = "AgentData",
                    TimeToWait = "DoJobTimer",
                    Gather = "GatheringData",
                    ShouldReceiveXp = "ShouldReceiveXp"
                },
                {
                    Name = "GoToWorkplaceFinish",
                    Type = "GO_TO",
                    AgentData = "AgentData",
                    Destination = "WorkplacePosition",
                    BuildingPathType = "",
                    AnimationData = "",
                    AnimationSpeedMultiplier = ""
                },
                {
                    Name = "EnablePaths",
                    Type = "ENABLE_PATH_TRACING",
                    AgentData = "AgentData"
                },
                {
                    Name = "ProduceResource",
                    Type = "PRODUCE_RESOURCE",
                    AgentData = "AgentData",
                    TimeToWait = "DepositResourceTimer",
                    ResourceProducedList = "ResourceProducedList",
                },
                {
                    Name = "GiveXp",
                    Type = "GIVE_JOB_XP",
                    AgentData = "AgentData",
                    ShouldReceiveXp = "ShouldReceiveXp"
                },
            },
        },
	}
})