local mod = ...



-- Hide the vanilla Well monument -------------------------------------------------------------------------------------
mod:overrideAsset({
    Id = "BUILDING_WELL",
    IsHidden = true,
})



-- Update the water resource -------------------------------------------------------------------------------------
-- Sell water and store in granaries
mod:overrideAsset({
    Id = "WATER",
    DisplayInInventory = true,
    DisplayInToolbar = true,
    OrderId = 0,
    VillagerBuyingPrice = {
        Resource = "GOLD_COINS",
        Quantity = 1,
    },
    ResourceVisualPrefab = "PREFAB_RESOURCE_VISUAL_CRATE",
})

mod:overrideAsset({
    Id = "BUILDING_FUNCTION_FOOD_MARKET",
    TypeList = {
        Action = "APPEND",
        "WATER",
    },
})

mod:overrideAsset({
    Id = "BUILDING_FUNCTION_GRANARY",
    AllowedResourceTypeList = {
        Action = "APPEND",
        {
            ResourceType = "WATER",
        },
    },
})



-- Assets -------------------------------------------------------------------------------------
mod:registerAssetProcessor("models/Well2.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })

mod:registerAssetId("models/Well2.fbx/Prefab/Well.Part.001", "PREFAB_IRRIGATION_WELL")
mod:registerAssetId("models/Well2.fbx/Prefab/Irrigation.Part.001", "PREFAB_IRRIGATION_TOWER")
mod:registerAssetId("models/Well2.fbx/Prefab/Digger.Part", "PREFAB_IRRIGATION_TOWER_DIGGER")
mod:registerAssetId("models/Well2.fbx/Prefab/Carrier.Part", "PREFAB_IRRIGATION_TOWER_CARRIER")



-- Grounding -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/Well2.fbx/Prefab/Well.Part.001/Attach_Major.Water", {DataType = "COMP_GROUNDED" })



-- Building Function: Well Water Generator -------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_RESOURCE_GENERATOR",
    Id = "BUILDING_FUNCTION_IRRIGATION_WELL",
	Name = "BUILDING_IRRIGATION_WELL_NAME",
    Description = "BUILDING_IRRIGATION_WELL_DESC",
    IsDescriptionOverride = false,
	ResourceGenerated = "WATER",
	IsForConsumer = true,
	IsInfinite = false,
})

local COMP_IRRIGATION_WELL = {
	TypeName = "COMP_IRRIGATION_WELL",
	ParentType = "COMPONENT",
	Properties = {
    }
}

function COMP_IRRIGATION_WELL:onEnabled()

    -- Set the resource generator
    local compResGen = self:getOwner():getOrCreateComponent("COMP_RESOURCE_GENERATOR")
    compResGen:setResourceGeneratorData("BUILDING_FUNCTION_IRRIGATION_WELL")

end

mod:registerClass(COMP_IRRIGATION_WELL)

mod:registerPrefabComponent("PREFAB_IRRIGATION_WELL", { DataType = "COMP_BUILDING_PART"})
mod:registerPrefabComponent("PREFAB_IRRIGATION_WELL", { DataType = "COMP_RESOURCE_GENERATOR", GrowRate = (1/10), MaxQuantity = 20 })
mod:registerPrefabComponent("PREFAB_IRRIGATION_WELL", { DataType = "COMP_IRRIGATION_WELL"})



-- Building: Well -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "BUILDING",
	Id = "BUILDING_IRRIGATION_WELL",
	Name = "BUILDING_IRRIGATION_WELL_NAME",
	Description = "BUILDING_IRRIGATION_WELL_DESC",
	BuildingType = "FOOD_PRODUCTION",
    AssetCoreBuildingPart = "BUILDING_PART_IRRIGATION_WELL",
	IsEditable = true,
})

mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_IRRIGATION_WELL",
	Name = "BUILDING_IRRIGATION_WELL_NAME",
	Description = "BUILDING_IRRIGATION_WELL_DESC",
    AssetBuildingFunction = "BUILDING_FUNCTION_IRRIGATION_WELL",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_IRRIGATION_WELL",
        MiniatureConfig = {
            ClipUnderGround = true,
            GroupHeight = 0.0,
            CameraPosition = { 0, 0, 0 },
            OrientationOffset = 0.0,
            CameraPitchOffset = 25.0
        },
	},
	BuildingZone = {
        ZoneEntryList = {
            {
                Polygon = polygon.createCircle( 1, {0, 0}, 7 ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true }
			},
            {
                Polygon = polygon.createCircle( 1, {0, -2.0}, 4 ),
                Type = { DEFAULT = false, GROUND_CONSTRAINT = true, NAVIGABLE = true, GRASS_CLEAR = false }
			},
            {
                Polygon = polygon.createCircle( 1, {0, -6.0}, 4 ),
                Type = { DEFAULT = false, WATER_CONSTRAINT = true, NAVIGABLE = false, GRASS_CLEAR = false }
			},
		},
    },
	Cost = {
        UpkeepCost = {
            { Resource = "GOLD_COINS", Quantity = 2 },
        },
        ResourceNeededList = {
            {
                { Resource = "STONE", Quantity = 4 },
                { Resource = "TOOL", Quantity = 2 },
            },
		},
	},
})



-- Component: Irrigation -------------------------------------------------------------------------------------
-- Tracks the water level
local COMP_IRRIGATION_TOWER = {
	TypeName = "COMP_IRRIGATION_TOWER",
	ParentType = "COMPONENT",
	Properties = {
        {
            Name = "WaterLevel",
            Type = "integer",
            Default = 0,
            Flags = { "SAVE_GAME" }
        },
    },
}

function COMP_IRRIGATION_TOWER:onEnabled()

    self:drawWater()

end

function COMP_IRRIGATION_TOWER:drawWater()

    local objTower = self:getOwner()
    local waterLevel = self.WaterLevel

    objTower:forEachChild(function(objChild)

        if objChild.Name == "Water" then

            local posTower = objTower:getGlobalPosition()
            objChild:setGlobalPosition({posTower[1], posTower[2] + (0.20 * waterLevel), posTower[3]})

        end

    end)

end

mod:registerClass(COMP_IRRIGATION_TOWER)

mod:registerPrefabComponent("PREFAB_IRRIGATION_TOWER", { DataType = "COMP_IRRIGATION_TOWER" })



-- Building: Irrigation -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "BUILDING",
	Id = "BUILDING_IRRIGATION_TOWER",
	Name = "BUILDING_IRRIGATION_TOWER_NAME",
	Description = "BUILDING_IRRIGATION_TOWER_DESC",
	BuildingType = "FOOD_PRODUCTION",
    AssetCoreBuildingPart = "BUILDING_PART_IRRIGATION_TOWER",
	IsEditable = true,
})

-- Building Part: Digger
mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_IRRIGATION_TOWER_DIGGER",
	Name = "BUILDING_FUNCTION_IRRIGATION_TOWER_DIGGER_NAME",
    AssetBuildingFunction = "BUILDING_FUNCTION_IRRIGATION_TOWER_DIGGER",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_IRRIGATION_TOWER_DIGGER",
	},
})

-- Building Part: Carrier
mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_IRRIGATION_TOWER_CARRIER",
	Name = "BUILDING_FUNCTION_IRRIGATION_TOWER_CARRIER_NAME",
    AssetBuildingFunction = "BUILDING_FUNCTION_IRRIGATION_TOWER_CARRIER",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_IRRIGATION_TOWER_CARRIER",
	},
})

-- Assemblage of diggers and carrier with the irrigation tower prefab
mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_IRRIGATION_TOWER_ASSEMBLAGE",
	Name = "BUILDING_IRRIGATION_TOWER_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
		CoreRandomBuildingPartList = { "PREFAB_IRRIGATION_TOWER" },
		MandatoryBuildingPartList = {
			{ BuildingPart = "BUILDING_PART_IRRIGATION_TOWER_CARRIER", OptionalAttachNodeString = "Attach_Major.001" },
			{ BuildingPart = "BUILDING_PART_IRRIGATION_TOWER_DIGGER", OptionalAttachNodeString = "Attach_Major.002" },
			{ BuildingPart = "BUILDING_PART_IRRIGATION_TOWER_DIGGER", OptionalAttachNodeString = "Attach_Major.003" },
		},
	},
})

-- Final part
mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_IRRIGATION_TOWER",
	Name = "BUILDING_IRRIGATION_TOWER_NAME",
    Description = "BUILDING_IRRIGATION_TOWER_DESC",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
        RandomPartList = {
            "BUILDING_PART_IRRIGATION_TOWER_ASSEMBLAGE",
        },
    },
    BuildingZone = {
        ZoneEntryList = {
            {
                Polygon = polygon.createRectangle( {2.5, 2.5}, {0, 0} ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true }
			},
		},
    },
	Cost = {
        UpkeepCost = {
            { Resource = "GOLD_COINS", Quantity = 8 },
        },
        ResourceNeededList = {
            {
                { Resource = "WOOD", Quantity = 4 },
                { Resource = "PLANK", Quantity = 4 },
                { Resource = "TOOL", Quantity = 3 },
            },
		},
	},
})



-- Building Function ---------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "NOTIFICATION",
    Id = "NOTIFICATION_IRRIGATION_NO_ZONE",
    Title = "NOTIFICATION_IRRIGATION_NO_ZONE_NAME",
    DelayBeforeHide = 30,
    IsCycling = true,
    DelayBetweenCycling = 60 * 7,
})

mod:registerAsset({
    DataType = "NOTIFICATION",
    Id = "NOTIFICATION_IRRIGATION_NO_RESOURCE",
    Title = "NOTIFICATION_IRRIGATION_NO_RESOURCE_NAME",
    DelayBeforeHide = 30,
    IsCycling = true,
    DelayBetweenCycling = 60 * 7,
})

mod:registerAsset({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_IRRIGATION_TOWER_DIGGER",
    Name = "BUILDING_FUNCTION_IRRIGATION_TOWER_DIGGER_NAME",
	Description = "BUILDING_FUNCTION_IRRIGATION_TOWER_DIGGER_DESC",
    WorkerCapacity = 1,
    RelatedJob = {
        Job = "JOB_IRRIGATION_TOWER_DIGGER",
        Behavior = "BEHAVIOR_IRRIGATION_DIG",
    },
    IsDescriptionOverride = false,
    InputInventoryCapacity = {
    },
    ResourceListNeeded = {
        { Resource = "RESOURCE_DESERT", Quantity = 1 },
    },
    ResourceProduced = {
    },
    AssetNoZoneNotification = "NOTIFICATION_IRRIGATION_NO_ZONE",
    AssetNoResourceInZoneNotification = "NOTIFICATION_IRRIGATION_NO_RESOURCE",
})

mod:registerAsset({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_IRRIGATION_TOWER_CARRIER",
    Name = "BUILDING_FUNCTION_IRRIGATION_TOWER_CARRIER_NAME",
	Description = "BUILDING_FUNCTION_IRRIGATION_TOWER_CARRIER_DESC",
    WorkerCapacity = 1,
    RelatedJob = {
        Job = "JOB_IRRIGATION_TOWER_CARRIER",
        Behavior = "BEHAVIOR_IRRIGATION_CARRY",
    },
    IsDescriptionOverride = false,
    InputInventoryCapacity = {
        { Resource = "WATER", Quantity = 5 },
    },
    ResourceListNeeded = {
        { Resource = "WATER", Quantity = 1 },
    },
    ResourceProduced = {
    },
})



-- Job -------------------------------------------------------------------------------------

-- Digger
mod:registerAsset({
	DataType = "JOB",
	Id = "JOB_IRRIGATION_TOWER_DIGGER",
	JobName = "BUILDING_FUNCTION_IRRIGATION_TOWER_DIGGER_NAME",
	JobDescription = "BUILDING_FUNCTION_IRRIGATION_TOWER_DIGGER_DESC",
	IsLockedByDefault = false,
	DefaultBehavior = "BEHAVIOR_IRRIGATION_DIG",
    ProductionDelay = 20,
	RelatedZone = "EXTRACTION",
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP",
		RightHandObject = {
			DataType = "HAND_OBJECT",
			Model = "PREFAB_TOOL_COAL_BURNER_SHOVEL",
		},
		WalkAnimation = "WALKING",
		IdleAnimation = "GOLDSMELTER_SHOVELING",
	},
})

mod:overrideAsset({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_IRRIGATION_TOWER_DIGGER"
    }
})

mod:overrideAsset({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_IRRIGATION_TOWER_DIGGER"
    }
})

-- Water carrier
mod:registerAsset({
	DataType = "JOB",
	Id = "JOB_IRRIGATION_TOWER_CARRIER",
	JobName = "BUILDING_FUNCTION_IRRIGATION_TOWER_CARRIER_NAME",
	JobDescription = "BUILDING_FUNCTION_IRRIGATION_TOWER_CARRIER_DESC",
	IsLockedByDefault = false,
	DefaultBehavior = "BEHAVIOR_IRRIGATION_CARRY",
    ProductionDelay = 5,
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP",
        LeftHandObject = {
            Model = "PREFAB_TOOL_DAIRY_FARMER_BUCKET",
        },
        RightHandObject = {
            Model = "PREFAB_TOOL_DAIRY_FARMER_BUCKET",
        },
		WalkAnimation = "WALKING",
		IdleAnimation = "IDLE",
	},
})

mod:overrideAsset({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_IRRIGATION_TOWER_CARRIER"
    }
})

mod:overrideAsset({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_IRRIGATION_TOWER_CARRIER"
    }
})

